/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (C) 2025 Amlogic, Inc. All rights reserved
 */
#ifndef _DT_BINDINGS_AMLOGIC_S7D_POWER_H
#define _DT_BINDINGS_AMLOGIC_S7D_POWER_H

#define PWRC_S7D_DOS_HCODEC_ID			0
#define PWRC_S7D_DOS_HEVC_ID			1
#define PWRC_S7D_DOS_VDEC_ID			2
#define PWRC_S7D_VPU_HDMI_ID			3
#define PWRC_S7D_USB_U2DRD_ID			4
#define PWRC_S7D_USB_U2H_ID			5
#define PWRC_S7D_SSD_EMMC_C_ID			6
#define PWRC_S7D_GE2D_ID			7
#define PWRC_S7D_AMFC_ID			8
#define PWRC_S7D_EMMC_A_ID			9
#define PWRC_S7D_EMMC_B_ID			10
#define PWRC_S7D_ETH_ID				11
#define PWRC_S7D_AUCPU_ID			12
#define PWRC_S7D_AUDIO_ID			13
#define PWRC_S7D_SRAMA_ID			14
#define PWRC_S7D_DMC0_ID			15
#define PWRC_S7D_DMC1_ID			16
#define PWRC_S7D_DDR_ID				17

#endif
