/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * nomadik.h
 *
 * Copyright (C) ST-Ericsson SA 2013
 * Author: Gabriel Fernandez <gabriel.fernandez@st.com> for ST-Ericsson.
 */

#define INPUT_NOPULL		0
#define INPUT_PULLUP		1
#define INPUT_PULLDOWN		2

#define OUTPUT_LOW		0
#define OUTPUT_HIGH		1
#define DIR_OUTPUT		2

#define SLPM_DISABLED		0
#define SLPM_ENABLED		1

#define SLPM_INPUT_NOPULL	0
#define SLPM_INPUT_PULLUP	1
#define SLPM_INPUT_PULLDOWN	2
#define SLPM_DIR_INPUT		3

#define SLPM_OUTPUT_LOW		0
#define SLPM_OUTPUT_HIGH	1
#define SLPM_DIR_OUTPUT		2

#define SLPM_WAKEUP_DISABLE	0
#define SLPM_WAKEUP_ENABLE	1

#define GPIOMODE_DISABLED	0
#define GPIOMODE_ENABLED	1

#define SLPM_PDIS_DISABLED	0
#define SLPM_PDIS_ENABLED	1
