/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides clock numbers for the ingenic,tcu DT binding.
 */

#ifndef __DT_BINDINGS_CLOCK_INGENIC_TCU_H__
#define __DT_BINDINGS_CLOCK_INGENIC_TCU_H__

#define TCU_CLK_TIMER0	0
#define TCU_CLK_TIMER1	1
#define TCU_CLK_TIMER2	2
#define TCU_CLK_TIMER3	3
#define TCU_CLK_TIMER4	4
#define TCU_CLK_TIMER5	5
#define TCU_CLK_TIMER6	6
#define TCU_CLK_TIMER7	7
#define TCU_CLK_WDT	8
#define TCU_CLK_OST	9

#endif /* __DT_BINDINGS_CLOCK_INGENIC_TCU_H__ */
