/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2022 Rockchip Electronics Co., Ltd.
 */
#ifndef __PHY_PCIE_H
#define __PHY_PCIE_H

#define PHY_MODE_PCIE_RC 20
#define PHY_MODE_PCIE_EP 21
#define PHY_MODE_PCIE_BIFURCATION 22

#endif
