/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides constants for the LM90 thermal bindings.
 */

#ifndef _DT_BINDINGS_THERMAL_LM90_H_
#define _DT_BINDINGS_THERMAL_LM90_H_

#define LM90_LOCAL_TEMPERATURE 0
#define LM90_REMOTE_TEMPERATURE 1
#define LM90_REMOTE2_TEMPERATURE 2

#endif
